from unixpackage.package_group import package_group_for_my_distro
from unixpackage.utils import log, warn, check_call


def install_command(generic_packages):
    """Return an install command for this distro."""
    return package_group_for_my_distro()(generic_packages).install_cmd()


def packages_installed(generic_packages):
    """Verify that the list of packages is installed."""
    if len(generic_packages) > 10:
        packages_installed(generic_packages[10:])
        generic_packages = generic_packages[:10]
    return package_group_for_my_distro()(generic_packages).check()


def install(generic_packages, polite=False):
    """Attempt installation of specified packages (if not already installed)."""
    if len(generic_packages) > 10:
        install(generic_packages[10:])
        generic_packages = generic_packages[:10]

    package_group = package_group_for_my_distro()(generic_packages)

    if not package_group.check():
        install_cmd = package_group.install_cmd()
        if package_group.need_sudo and polite:
            log((
                "The following command must be run to continue:\n"
                "  ==> {0}\n"
                "If you like, you can copy and paste the command "
                "and run it in another terminal window.\n"
                "After attempting to run the command "
                "unixpackage will verify successful installation.\n"
            ).format(install_cmd))
        try:
            check_call(install_cmd, shell=True)
        except exceptions.CalledProcessError:
            warn("\nWARNING : Command '{0}' returned error code\n\n".format(install_cmd))

        # Double check that the packages were correctly installed
        if not package_group.check():
            not_yet_installed = []
            for package in generic_packages:
                package_group_of_one = package_group_for_my_distro()([package])

                if not package_group_of_one.check():
                    not_yet_installed.append(package)

            # Throw meaningful error with the command to re-run to install packages
            not_yet_installed_package_group = package_group_for_my_distro()(not_yet_installed)
            if len(not_yet_installed) > 0:
                raise exceptions.PackageInstallationFailed(
                    not_yet_installed,
                    not_yet_installed_package_group.install_cmd()
                )
    else:
        log("Already installed: {0}\n".format(', '.join(generic_packages)))



from subprocess import call, Popen, PIPE
from unixpackage import exceptions
import requests
import json
import io


def return_code_zero(command):
    """Returns True if command called has return code zero."""
    return call(command, stdout=PIPE, stderr=PIPE) == 0

def check_output(command, stdout=PIPE, stderr=PIPE):
    """Re-implemented subprocess.check_output since it is not available < python 2.7."""
    return Popen(command, stdout=stdout, stderr=stderr).communicate()[0]

def check_call(command, shell=True):
    """Re-implemented subprocess.check_call since it is not available < python 2.7."""
    process = Popen(command, shell=shell)
    process.communicate()
    if process.returncode != 0:
        raise exceptions.CalledProcessError
    return

def is_string(obj):
    """Is the object a string/unicode?"""
    return str(type(obj)) == "<type 'unicode'>" or str(type(obj)) == "<class 'str'>"

def get_json_from_file(cache_filename):
    with open(cache_filename, 'r') as cache_read_handle:
        return json.loads(cache_read_handle.read())

def save_json_to_file(cache_filename, contents):
    with open(cache_filename, 'w') as cache_write_handle:
        cache_write_handle.write(json.dumps(contents))

def get_request(url):
    try:
        req = requests.get(url)
    except requests.exceptions.ConnectionError:
        raise exceptions.ConnectionFailure((
            "Failure when connecting to {0} "
            "Is your internet working?"
        ).format(url))
    return req

def _write(handle, message):
    if isinstance(handle, io.TextIOWrapper):
        handle.write(message)
    else:
        handle.write(message.encode('utf8'))
    handle.flush()

def log(message):
    """Output to stdout."""
    import sys
    _write(sys.stdout, message)

def warn(message):
    """Output to stderr."""
    import sys
    _write(sys.stderr, message)
